/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Stefan Lienhard
   co-programmer: ...
*/

#include "bomb.h"
#include "glincl.h"
#include "state.h"
#include "list.h"
#include "model.h"
#include "vector.h"
#include "fast_factory.h"


#include "particle_engine.h"
#include "particle_emitter.h"
#include "particle_system.h"

using namespace std;

CREATE_FAST_FACTORY_STATIC(Bomb, CL_BOMB);

/**
 * constructs and loads a Bomb from a XML-element
 * @param root the XML-element to load from
 */
Bomb::Bomb(const TiXmlElement* root)
{
  this->init();
  if (root != NULL)
    this->loadParams(root);

  float modelSize = 1.0;
  this->loadModel("models/projectiles/RadioActiveBomb.obj", 1.0);

  this->energyMin = 1;
  this->energyMax = 1;
  this->remove();
  this->lifeSpan = 15;

  this->emitter = new ParticleEmitter(Vector(0,1,0), M_2_PI, 100, 5);
  this->emitter->setParent(this);
  this->emitter->setSpread(M_PI, M_PI);
}


/**
 * standard deconstructor
 */
Bomb::~Bomb ()
{

}


/**
 * initializes the Bomb
 * @todo change this to what you wish
 */
void Bomb::init()
{
  this->setClassID(CL_BOMB, "Bomb");

  /**
   * @todo: Write CL_PROTO_WORLD_ENTITY INTO THE src/defs/class_id.h (your own definition)
   */

}


/**
 * loads a Bomb from a XML-element
 * @param root the XML-element to load from
 * @todo make the class Loadable
 */
void Bomb::loadParams(const TiXmlElement* root)
{
  // all the clases this Entity is directly derived from must be called in this way, to load all settings.
  static_cast<WorldEntity*>(this)->loadParams(root);


  /**
   * @todo: make the class Loadable
   */
}


/**
 * advances the Bomb about time seconds
 * @param time the Time to step
 */
void Bomb::tick(float time)
{
  //Vector v = *this->flightDirection * ( this->speed * time * 1000 + 0.1);
  Vector v = this->velocity * (time);
  this->shiftCoor(v);

  this->lifeCycle += time/this->lifeSpan;
  if( this->lifeCycle >= 1.0)
    {
      PRINTF(5)("FINALIZE==========================\n");
      PRINTF(5)("current life cycle is: %f\n", this->lifeCycle);
      PRINTF(5)("FINALIZE===========================\n");

      this->deactivate();
    }
}

/**
 * draws this worldEntity
 */
void Bomb::draw () const
{
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  float matrix[4][4];

  /* translate */
  glTranslatef (this->getAbsCoor ().x,
                this->getAbsCoor ().y,
                this->getAbsCoor ().z);
  /* rotate */
  this->getAbsDir().matrix(matrix);
  glMultMatrixf((float*)matrix);

  if (model)
    model->draw();
  glPopMatrix();
}


/**
 *
 *
 */
void Bomb::collidesWith (WorldEntity* entity, const Vector& location)
{
	this->detonate();
}

void Bomb::activate()
{
  State::getWorldEntityList()->add(this);

}

void Bomb::deactivate()
{
  State::getWorldEntityList()->remove(this);
  Bomb::fastFactory->kill(this);
}

void Bomb::detonate()
{
  tIterator<WorldEntity>* it = State::getWorldEntityList()->getIterator();
  WorldEntity* lm = it->firstElement();

  while(lm != NULL)
  {
   
    lm = it->nextElement();
  }
}
